# Copyright (C) 2010 Paraguay Educa, Martin Abente, Bernie Innocenti
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  US

import os
import gobject
import glib
import gio
import logging

BYTES_TO_READ = 100

class JournalManagementModel(gobject.GObject):

    __gtype_name__ = 'JournalManagementModel'

    __gsignals__ = {
        'journal-management-running'    : (gobject.SIGNAL_RUN_FIRST, gobject.TYPE_NONE, ([str])),
        'journal-management-started'    : (gobject.SIGNAL_RUN_FIRST, gobject.TYPE_NONE, ([])),
        'journal-management-finished'    : (gobject.SIGNAL_RUN_FIRST, gobject.TYPE_NONE, ([])),
        'journal-management-failed'    : (gobject.SIGNAL_RUN_FIRST, gobject.TYPE_NONE, ([]))
    }

    def __init__(self):
        gobject.GObject.__init__(self)

        self._running = False
        self._backup_script_path = ['/usr/bin/journal-backup']
        self._restore_script_path = ['/usr/bin/journal-restore']

    def do_backup(self):
        self.run_cmd_async(self._backup_script_path)

    def do_restore(self):
        self.run_cmd_async(self._restore_script_path)

    def report_process_status(self, stream, result):
        data = stream.read_finish(result)

        if len(data):
            self.emit('journal-management-running', data)
            stream.read_async(BYTES_TO_READ, self.report_process_status)

    def run_cmd_async(self, cmd):
        if self._running == False:
            try:
                pid, stdin, stdout, stderr = glib.spawn_async(cmd, flags=glib.SPAWN_DO_NOT_REAP_CHILD, standard_output=True)
                gobject.child_watch_add(pid, _handle_process_end, (self))
            except Exception:
                self.emit('journal-management-failed')
            else:
                stdin_stream = gio.unix.InputStream(stdout, True)
                stdin_stream.read_async(BYTES_TO_READ, self.report_process_status)

                self._running  = True
                self.emit('journal-management-started')

def _handle_process_end(pid, condition, myself):
    myself._running = False

    if os.WIFEXITED(condition) and\
        os.WEXITSTATUS(condition) == 0:
            myself.emit('journal-management-finished')
    else:
        myself.emit('journal-management-failed')

